function plotWeigthCourse(course, figstring, titlestring)
%Diese Funktion generiert eine oder mehrere Figures in denen der
%Verlauf der Gewichte beim Lernen des Modells abgetragen ist.
%Übergeben werden eine Matrix (course) mit den Gewichten zu
%verschiedenen Zeitpunkten (jede Zeile ein Zeitpunkt) und einen
%Parameter der die Gesamtzahl der Trials angibt (trials) sowie einen
%Parameter der angibt in welchen Abständen die Gewichte protokolliert
%wurden (step).Letzterer scheint nötig um bei hohen Trialzahlen
%den Speicherbedarf in vertretbarem Rahmen zu halten (s. auch rescWagn.m).
%Der Übersichtlichkeit halber werden in einer einzigen Figure nicht mehr
%als 12 Gewichte auf einmal dargestellt. Besteht das Modell aus mehr als
%12 Input-Units, so werden die ersten 12 in der ersten Figure, die zweiten
%12 in der zweiten Figure und so weiter abgetragen.

% %Damit die Sachen auch auf einem S/W-Drucker halbwegs manierlich aussehen:
% set(0,'DefaultAxesColorOrder',[0 0 0],'DefaultAxesLineStyleOrder','-|--|:')


% Copyright (C) 2006-2008  Anna Thorwart and Holger Schultheis


step=1;
[r, c] = size(course);

%Wieviele Figures brauchen wir?
noFig = ceil(c/12);
%figHandles = [];

%für jede Figure
for i = 1:noFig
    %Bestimmung der Gewichte die aktuell zu plotten sind
    lowerBound = (i - 1)*12 + 1;
    upperBound = min(i * 12, c);
    %Titel der aktuellen Figure
%     figtitle = strcat(figstring, 'weights ', num2str(lowerBound), ' to ', num2str(upperBound));
    figtitle = [figstring, 'weights  ', num2str(lowerBound), ' to ', num2str(upperBound)];
    %Gewichtbezeichnung für die Legende
    lW = {};
    iter = 1;
    for j = lowerBound:upperBound
        lW(iter, 1) = {strcat('W', num2str(j))};
        iter = iter + 1;
    end
    lW = char(lW);
    %Selektion der Teilmenge der Gewichte
    tmp = course(:, lowerBound:upperBound);
    %figHandles = [figHandles, figure('NumberTitle', 'off', 'Name', title)];
    
    %Jetzt die Figure erstellen...
    figure('NumberTitle', 'off', 'Name', figtitle);
    hold;
    %...und die Gewichtsverläufe zeichnen.
    if (upperBound - lowerBound) < 4
        plot(tmp(:, 1:end), 'Marker', 'x');
    else
        plot(tmp(:, 1:3), 'Marker', 'x');
        plot(tmp(:, 4:end), 'Marker', 'o');
    end
    hold;
    %Zum Schluss noch ein bißchen Kosmetik damit die Figure ansehnlicher
    %wird.
    xlabel('trials'), ylabel('associative weights');
    title(titlestring)
    legend(lW);
    autoXTicks = get(gca, 'XTick');
    %Diese Art der XTickLabel-Berechnung ist nicht ganz unproblematisch scheint
    %aber für die hiesigen Zwecke ausreichend (s. rescWagn).
    xtLabel = (autoXTicks  - 1) * step;
    set(gca, 'XTickLabel', xtLabel);
    
end