function [CS, US] = randsequ(Block, nBlock,maxUS, Kont, randomOrder,Vp)
% generates stimulus sequences

% Input
% Block:        CS that occur during training , specified by their line number in Stimulusname
% maxUS:          max. number of following trials with the same contingency
% nBlock:       no of blocks
% Kont:         US paired with CS
% randOrder:    randomize sequences?
% Vp:           number of subjects

% Ouptut:
% CS:           CS-sequence: noTrial*AnzVp-matrix
% US:           US/contingencies-sequence: noTrial*AnzVp-matrix

% Copyright (C) 2006  Stephan Koenig and Anna Thorwart

%% 1. method
noPhases=size(Block,1);
if noPhases ~= size(nBlock,1)
    errordlg('Different numbers of phases are defined in "number of blocks" and in "training block" .')
    CS=[];
    US=[];
    return
end


for pp=1:noPhases
    for ii = 1:Vp;
        
        Reize=repmat(Block{pp}, nBlock(pp), 1); %replicate block
        if max(Reize)>size(Kont,1)
            errordlg(['There are trials specifed in Phase ' num2str(pp) ' that are not defined in the trial matrix.'])
            CS=[];
            US=[];
            return
        end
        
        if randomOrder %randomize: randomize with the contraint that only "max." following trials have the same contingency, i.e.
            
            %% Set random seed
            %   generate a random stream from the computer clock
            if verLessThan('matlab', '7.13')
                s = RandStream.create('mt19937ar','seed',sum(100*clock));
                RandStream.setDefaultStream(s);
                clear s
            else
                rng('shuffle')
            end;
            
            
            if isempty (maxUS)|| isnan(maxUS)
                errordlg('"Max. number of following trials with the same lambda" is missing.')
                CS=[];
                US=[];
                return
            end
            if exist('figh','var') && isgraphics(figh) %isvalid(figh)
                set(findobj(figh,'Tag','MessageBox'),'String',{['Randomizing Phase ' num2str(pp) ' of sequence no. ' num2str(ii) '.']; 'Press button to abort before finishing.'})
            else
                figh = msgbox({['Randomizing Phase ' num2str(pp) ' of sequence no. ' num2str(ii) '.']; 'Press button to abort before finishing.'} );
            end
            
            outcome=unique(Kont);
            blockOut = Kont(Block{pp});
            
            
            %take a first shot
            CSVP=Reize(randperm(size(Reize,1)),:);
            USVP=Kont(CSVP);
            
            %check and when necessary correct
            for currentCont=outcome';
                pattern=ones(1,maxUS+1)*currentCont;
                test= (strfind(USVP', pattern));
                while ~isempty(test) && isgraphics(figh) %isvalid(figh)
                    %generate new sequence while there are still parts that
                    %don't meet the restriction and the user has not
                    %aborted the randomisation yet
                    pause(0.000000001);
                    CSVP=Reize(randperm(size(Reize,1)),:);
                    USVP=Kont(CSVP);
                    test=(strfind(USVP', pattern));
                end
                if ~isgraphics(figh) %isvalid(figh)
                    errordlg('You aborted the sequence randomisation. No sequences were generated.')
                    CS=[];
                    US=[];
                    return
                end
            end
            
            CSphase{pp,1}(:,ii)=CSVP;
            USphase{pp,1}(:,ii)=USVP;
        else
            CSphase{pp,1}(:,ii)=Reize;
            USphase{pp,1}(:,ii)=Kont(Reize);
        end
    end
    
end

%combine phases
CS=cell2mat(CSphase);
US=cell2mat(USphase);

if exist('figh', 'var')
    close(figh)
end
