function paramsINM=trainINM(paramsGen, paramsINM)
%simulation, ploting and saving of the Inhibted Elements Model


% The generela parameters passed by paramsGen are
% Stimulusname: string name of all CS
% Testname:     "
% Kont:         US paired with CS
% KontTest:     "
% beta:         beta of USs
% AnzVp:        number of subjects
% CS:           CS-sequence: noTrial*AnzVp-matrix
% US:           US-sequence: noTrial*AnzVp-matrix
% code:         stimuluscode of all CS: a line defines the occurrence of each single stimulus in a CS (without unique cue or similar)
% testcode:     "
% block:        CS that occur during training , specified by their line number in Stimulusname

% paramsINM         structure with PARAMETER for Inhibited Elements Model
% resultsPath:      directory where resulting matrices are saved
% startingWeights:  txt-file with associative weights whose last line is used as starting
% alpha:            vector with alpha parameter
% nn:               amount of normalisation
% pattern:          expanded stimulus code
% noWeights:        number of elements needed to represent all stimuli combinations
% rule:             learning rule of Schultheis et al or Glautier?
% weightPlot:       checkbox for weightPlot
% activationPlots:  checkbox for activatiaonPlots

% Variables
% code:             stimulus code
% weights:          associative weights
% amatrix:          output activation by presented CS
% TestAktivierung   output activation by every CS

% Copyright (C) 2010  Anna Thorwart

% check parameter and prepare matrices
if isempty(paramsINM)
    errordlg('Something is missing.')
    return
end

noPattern=size(paramsGen.code,1);
noStimuli=size(paramsGen.code,2);
noTrials = size(paramsGen.CS,1);


% check alphas
NoAlphas_theo=noStimuli; %number of alphas nedded
NoAlphas_be=size(paramsINM.alpha,2); %number of alphas entered

if NoAlphas_theo ~=NoAlphas_be
    meldung=['The number of alphas doesn''t fit. There should be ' num2str(NoAlphas_theo) ', but there are ' num2str(NoAlphas_be) '. Do you wish to run the simulation using the default value for all alphas?.'];
    answer= questdlg(meldung, 'Miscount!', 'Yes', 'Cancel', 'Yes');
    if strcmp('Yes', answer);
        paramsINM.alpha=repmat(paramsINM.alphadefault, 1, NoAlphas_theo);
    else
        return
    end
end

% stimulus representation

% compute number of units necesarry to represent each stimulus
elemts=ff2n(noStimuli); % all combinations of stimuli
elemts=elemts(elemts(:,1)==1,:); % delete those where first stimulus, i.e. the stimulus the units belong, is NOT activated
StringLength=sum(elemts,2); % number of stimuli coded in the unit
paramsINM.noWeights=size(elemts,1);

% compute activation (proportion) of units inhibited by stimuli combinations consisting of different number of stimui (=StringLength)
% in representation of single stimulus so that the
% overall activation of all stimuli corresponds to the amount of normalisation specified with paramsINM.nn When nn=1, the activation is always 1(complete
% normalisation). When nn=0, the first unit, that is never inhibited, is
% completely activated to 1.
proportions=zeros(1,paramsINM.noWeights);
for jj=1:noStimuli
    proportions(StringLength==jj)= 1/(noStimuli-(jj-1))^paramsINM.nn - sum(proportions(~any(elemts(:,2:(noStimuli+1-jj))==1,2)));
end


% expand code based on computed proportions and stimulus pattern presented
% in experiment
paramsINM.pattern=zeros(noPattern, paramsINM.noWeights*noStimuli);

for jj= 1:noPattern
    for kk=1:noStimuli
        if paramsGen.code(jj,kk)
            patt = [paramsGen.code(jj,kk) paramsGen.code(jj,1:(kk-1)) paramsGen.code(jj,(kk+1):noStimuli)];
            % activation of this CS in this pattern & activation of remaining, ie. context, stimuli in this patternactTemp=[];
            indCS=find(patt>0);
            paramsINM.pattern(jj,(kk-1)*paramsINM.noWeights+1 : kk*paramsINM.noWeights)=proportions.*(~any(elemts(:,indCS(2:end))==1,2))';
        end
    end
    
end

paramsINM.testpattern=paramsINM.pattern;
noT = size(paramsINM.testpattern, 1);
disp('simulating model...')

noInp = paramsINM.noWeights*noStimuli;
alpha=reshape(repmat(paramsINM.alpha,paramsINM.noWeights,1),1,noInp); %expand alpha

meanWeights=zeros(noTrials+1, noInp);

% output activation by each pattern in each trial
TestActivation(noTrials,noPattern,paramsGen.AnzVp)=0;
% output activation by pattern presented in that trial
Output=zeros(noTrials, paramsGen.AnzVp);


for s=1:paramsGen.AnzVp
    
    
    %%Anfangswerte setzen
    path=paramsINM.startingWeights;
    % associative weights
    if isempty(path)
        Ass_Str = zeros(1, noInp);
    else % TODO: im Moment fuer jede Vp die gleichen Startgewichte
        Ass_Str=textread(path, '%s', 'bufsize', 100000095, 'delimiter','\n', 'whitespace',' ');
        Ass_Str=str2num(char(Ass_Str(size(Ass_Str,1),:)));
    end
    
    
    for jj=1:noTrials
        TestActivation(jj,:,s)=(paramsINM.pattern*Ass_Str(jj,:)')';
        Output(jj,s)=paramsINM.pattern(paramsGen.CS(jj,s),:)*Ass_Str(jj,:)';
        error=paramsGen.US(jj,s)-Output(jj,s);
        beta = paramsGen.beta(paramsGen.CS(jj,s));
        Ass_Str(jj+1,:)=Ass_Str(jj,:)+(logical(paramsINM.pattern(paramsGen.CS(jj,s),:))'.*alpha'.*beta*(error))';
        
    end
    
    meanWeights=meanWeights+Ass_Str;
    OutputVp=Output(:,s)';
    amatrix(:,:,s)=sorttrials(OutputVp, paramsGen.CS(:,s), length(paramsGen.Stimulusname));
end
amatrix(:,:,s+1)=mean(amatrix,3);

TestActivation(:,:,s+1)=mean(TestActivation,3);
meanWeights= meanWeights/paramsGen.AnzVp;

%% ploting

%create string with training stimulus
trainingstring=[];
for ii=1:length(paramsGen.block)
    trainingstring = [trainingstring, strjoin(paramsGen.Stimulusname(paramsGen.block{ii})',  ','),'|'];
end

% create string with parameters

if length(paramsGen.beta)<=7
    titlestring ={['normalisation: ', num2str(paramsINM.nn),';'];...
        ['\alpha=', mat2str(paramsINM.alpha'),'; \beta=', mat2str(paramsGen.beta),'; phases: ', trainingstring]};
else
    titlestring ={['learning rule: ', num2str(paramsINM.nn),';'];...
        ['\alpha=', mat2str(paramsINM.alpha'),'; \beta=xxx ; phases: ', trainingstring]};
end

if paramsINM.weightPlot
    plotWeigthCourse(meanWeights(:,:),'associative weights of elements: inhibited elements model, ', titlestring);
end

if paramsINM.activationPlots
    plotOutputActivation(TestActivation(:,:,s+1),'activation by teststimuli: inhibited elements model', titlestring,paramsGen.Stimulusname );
    %     plotOutputActivation(amatrix(:,:,s+1),'activation by presented stimulus: inhibited', titlestring, paramsGen.Stimulusname);
end

%% saving

speicherpath=paramsINM.resultsPath;

if ~isempty(speicherpath{1});
    dlmwrite ([speicherpath{1}, '\',speicherpath{2}, '_testactivation.txt'], TestActivation(:,:,s+1), '\t') ;
    dlmwrite ([speicherpath{1}, '\',speicherpath{2},'_outputactivation.txt'], amatrix(:,:,s+1), '\t')
    dlmwrite ([speicherpath{1}, '\',speicherpath{2}, '_weights.txt'],meanWeights(:,:),'\t') ;
else
    disp('Nothing is saved as there is no ouptut location specified!')
end

