function paramsP=trainP(paramsGen, paramsP)
% simulation, ploting and saving of the Pearce Model and its extension

% The generela parameters passed by paramsGen are
% Stimulusname: string name of all CS
% Testname:     "
% Kont:         US paired with CS
% KontTest:     "
% beta:         beta of USs
% AnzVp:        number of subjects
% CS:           CS-sequence: noTrial*AnzVp-matrix
% US:           US-sequence: noTrial*AnzVp-matrix
% code:         stimuluscode of all CS: a line defines the occurrence of each single stimulus in a CS (without unique cue or similar)
% testcode:     "
% block:        CS that occur during training , specified by their line number in Stimulusname



% paramsP:          parameter for Pearce simulation
% d                 generalization parameter (Kinder & Lachnit, 2001)
% resultsPath:      directory where resulting matrices are saved
% startingWeights:  txt-file with associative weights whose last line is used as starting
% alpha             vector with alpha parameter for each configural unit
% alphadefault      default value for alpha definded in default.conf
% weightPlot:       checkbox for weightPlot
% activationPlots:  checkbox for activatiaonPlots

% Variables
% code:             stimulus code (at the moment: =testcode, =Vgl)
% weights:          associative weights of all subjects


% Copyright (C) 2006-2008  Anna Thorwart and Holger Schultheis

%check alpha for input units;
NoAlphas_theo=size(paramsGen.code,2); %number of alphas necessary
NoAlphas_be=size(paramsP.alphasinput,2); %number of alphas specified
if NoAlphas_theo ~=NoAlphas_be
    meldung=['The number of alphas for input units doesn''t fit. There should be ' num2str(NoAlphas_theo) ', but there are ' num2str(NoAlphas_be) '. Do you wish to run the simulation using the default value for all alphas?'];
    answer= questdlg(meldung, 'Miscount!', 'Yes', 'Cancel', 'Yes');
    if strcmp('Yes', answer);
        paramsP.alphasinput=repmat(paramsP.alphadefault, 1, NoAlphas_theo);
        
    else
        return
    end
end

%weight stimulus code wiht alphas
code=paramsGen.code.*repmat(paramsP.alphasinput,[size(paramsGen.code,1) 1]);  %activation of input units

%test stimuli
testcode=code;
noT=size(testcode,1);


%check alpha for configural units;
NoAlphas_theo=size(paramsGen.code,1); %number of alphas necessary
NoAlphas_be=size(paramsP.alpha,2); %number of alphas specified
if NoAlphas_theo ~=NoAlphas_be
    meldung=['The number of alphas for the configural units doesn''t fit. There should be ' num2str(NoAlphas_theo) ', but there are ' num2str(NoAlphas_be) '. Do you wish to run the simulation using the default value for all alphas?'];
    answer= questdlg(meldung, 'Miscount!', 'Yes', 'Cancel', 'Yes');
    if strcmp('Yes', answer);
        paramsP.alpha=repmat(paramsP.alphadefault, 1, NoAlphas_theo);
        
    else
        return
    end
end
alpha_reiz = paramsP.alpha;
SIMPAR=paramsP.d;

%Vergleichs-Matrix fuer die Aktivierung der Konfigural-Units=Aktivierte Input-Units=Elemente
Vgl=code; %activation of input units
KonfigUnits=size(Vgl,1); %number of configural units
 
 
% initialise matrix for associative weights  of all subjects
weights=zeros(size(paramsGen.CS,1),size(Vgl,1), paramsGen.AnzVp); 


%% for each subject
for s=1:paramsGen.AnzVp
    
    CScodeVp=code(paramsGen.CS(:,s),:);
    USVP=paramsGen.US(:,s);
    
    % starting weights
    if isempty(paramsP.startingWeights)
        Ass_Str =zeros(size(Vgl,1),1);
    else
        path = paramsP.startingWeights;
        startingWeights=dlmread(path); %read in weights matrix
        Ass_Str= startingWeights(end,:)' ;% set last line as starting weights
        weights(1,:,s)=startingWeights(end,:); % write starting weights in weights matrices
    end
           
  
    Trials=size(CScodeVp,1); %number of trials
   
    
    % learning
    for j=1:Trials
        normCScodeVp=CScodeVp(j,:)'./norm(CScodeVp(j,:)');  %activation of output units by presented CS in trial j
        for k=1:KonfigUnits
            normVgl=Vgl(k,:)'./norm(Vgl(k,:)'); %activation of output units by CS k (=connection strength between output and config unit k)
            simP(k)=(normCScodeVp'*normVgl)^SIMPAR;  %activation of configural unit k  by presented CS in trial j
        end
        Output(j,s)=simP*Ass_Str; %activation of the US unit by presented CS in trial j of sequence/subject s
        Fehler=USVP(j)-Output(j,s); %error
        beta = paramsGen.beta(paramsGen.CS(j,s));
        alpha = alpha_reiz(paramsGen.CS(j,s));
        
        for t=1:noT;
            simPTest=[];
            normtestcode=testcode(t,:)'./norm(testcode(t,:)');%activation of output units by test trial k (=connection strength between output and config units)
            for k=1:KonfigUnits;
                normVgl=Vgl(k,:)'./norm(Vgl(k,:)');
                simPTest(k)=(normtestcode'*normVgl)^SIMPAR;  %activation of configural unit k  by test trial t
            end
            TestAktivierung(j,t,s)=simPTest*Ass_Str; %activation of the US by CS in test trial t at the beginning of trial j
        end

        
        
        Selektion=zeros(size(simP));
        Selektion(paramsGen.CS(j,s))=1; %erstellt einen Vektor, der angibt, welche Konfigural-Unit lernt
        
        %NOTE: if  exactly the same stimulus is used in several different trial types(A+, A-, A?), there is
        %an unit for each trial type within the simulation. It even doesn't matter if these trials
        %are training or only test trials.
        %As all are completely activated in appropiate trials, there is perfect "generalisation"
        %of their already learned associations (Output computations at line 105).
        %However, only the associative weight of the presented trial type unit is changed.
        %This was different in ALTSim versions lower than 3.5. There, associative weights of all
        %units whose activation was 1 changed . Therefore, learning was unappropriatly accelerated for
        %those stimuli presented in several different trial types.
        
        Ass_Str=Ass_Str+(Selektion'*alpha*beta*(Fehler));
        weights(j+1,:,s)=Ass_Str';
    end
          OutputVp=Output(:,s)';
    
          amatrix(:,:,s)=sorttrials(OutputVp, paramsGen.CS(:,s), length(paramsGen.Stimulusname));
    
    
    %sum up the associatve weights of units belonging to the same CS
    [b, m, n] = unique(Vgl,'rows'); %get all unique stimuli
    weightsnew=zeros(size(weights,1),size(b,1));
    for ii=1:size(b,1) %for all stimuli
        weightsnew(:,ii)=sum(weights(:,n==ii,s),2); %sum up the associative strength for identical stimuli
        legNames{ii,:}=[paramsGen.Stimulusname{n==ii}];
    end
end

%% average over subjects
amatrix(:,:,s+1)=mean(amatrix,3);
TestAktivierung(:,:,s+1)=mean(TestAktivierung,3);
weights(:,:,s+1)= mean(weights,3);
weightsnew(:,:,s+1)= mean(weightsnew,3);



%% plotting

%create string with training stimulus
trainingstring=[];
for ii=1:length(paramsGen.block)
    trainingstring = [trainingstring, strjoin(paramsGen.Stimulusname(paramsGen.block{ii})', ','),'|'];
end

% create string with parameters

if size(paramsP.alpha,2)<7 & length(paramsGen.beta) < 7
    titlestring ={['d = ', num2str(paramsP.d), '; {\alpha}_{input}=', mat2str(paramsP.alphasinput'),';'];...
        ['\alpha_{configural}=', mat2str(paramsP.alpha'),'; \beta=', mat2str(paramsGen.beta), '; phases: ', trainingstring]};
elseif size(paramsP.alpha,2) > 7 & length(paramsGen.beta) <= 7
    titlestring ={['d = ', num2str(paramsP.d),  '; {\alpha}_{input}=', mat2str(paramsP.alphasinput'),';'];...
        ['\alpha_{configural}=xxx ; \beta=', mat2str(paramsGen.beta), '; phases: ', trainingstring]};
elseif size(paramsP.alpha,2)<=7 && length(paramsGen.beta) > 7
    titlestring ={['d = ', num2str(paramsP.d),  '; {\alpha}_{input}=', mat2str(paramsP.alphasinput'),';'];...
        ['\alpha_{configural}=', mat2str(paramsP.alpha'),'; \beta=xxx ; phases: ', trainingstring]};
else
    titlestring ={['d = ', num2str(paramsP.d), ';'];...
        ['\alpha_{configural}=xxx; \beta=xxx ; phases: ', trainingstring]};
end



if paramsP.weightPlot
%     plotWeigthCourse(weights(:,:,s+1),'associative weights of configural units: Pearce, ', titlestring);
%     legend(paramsGen.Stimulusname)
    plotWeigthCourse(weightsnew(:,:,s+1),'associative weights of configural units: Pearce, ', titlestring);
    legend(legNames)
end
if paramsP.activationPlots
    plotOutputActivation(TestAktivierung(:,:,s+1),'activation by teststimuli: Pearce ', titlestring,paramsGen.Stimulusname );
%           plotOutputActivation(amatrix(:,:,s+1),'activation by presented stimulus: Pearce ', titlestring, paramsGen.Stimulusname);
end

%% Saving
speicherpath=paramsP.resultsPath;
if ~isempty(speicherpath{2});
    dlmwrite ([speicherpath{1},'\', speicherpath{2}, '_testactivation.txt'], TestAktivierung(:,:,s+1), '\t') ;
       dlmwrite ([speicherpath{1},'\', speicherpath{2}, '_outputactivation.txt'], amatrix(:,:,s+1), '\t')
    dlmwrite ([speicherpath{1},'\', speicherpath{2}, '_weights.txt'],weights(:,:, s+1),'\t') ;
end
