function paramsREM=trainREM(paramsGen, paramsREM)
%simulation, ploting and saving of the Replaced Elements Model


% The generela parameters passed by paramsGen are
% Stimulusname: string name of all CS
% Testname:     "
% Kont:         US paired with CS
% KontTest:     "
% beta:         beta of USs
% AnzVp:        number of subjects
% CS:           CS-sequence: noTrial*AnzVp-matrix
% US:           US-sequence: noTrial*AnzVp-matrix
% code:         stimuluscode of all CS: a line defines the occurrence of each single stimulus in a CS (without unique cue or similar)
% testcode:     "
% block:        CS that occur during training , specified by their line number in Stimulusname

% paramsREM:         parameter for Replaced Elements Model
% resultsPath:      directory where resulting matrices are saved
% startingWeights:  txt-file with associative weights whose last line is used as starting
% alpha             vector with alpha parameter
% proportionPath    txt-file with replaced proportions
% pattern:          expanded stimulus code
% noWeights:        number of elements needed to represent all stimuli combinations
% rule:             learning rule of Schultheis et al or Glautier?
% weightPlot:       checkbox for weightPlot
% activationPlots:  checkbox for activatiaonPlots

% Variables
% code:             stimulus code
% weights:          associative weights 
% amatrix:          output activation by presented CS
% TestAktivierung   output activation by every CS

% Copyright (C) 2006-2008  Anna Thorwart and Holger Schultheis



% check parameter and prepare matrices
paramsREM= initEstes(paramsGen, paramsREM);
if isempty(paramsREM)
    errordlg('Something is missing.')
    return
end


assignin('base', 'pattern', paramsREM.pattern)


disp('simulating model...')


%Anzahl der Lerndurchgï¿½nge, Muster, Input-Units und Lernparameter holen
noPat = size(paramsGen.CS,1); %trials
noInp = paramsREM.noWeights;
noT = size(paramsREM.testpattern, 1);
weights = zeros(noPat, noInp);
meanWeights=zeros(noPat+1, noInp);





for s=1:paramsGen.AnzVp

    rpatternVp = paramsREM.pattern(paramsGen.CS(:,s),:);
    USVP=paramsGen.US(:,s);
    
    %%Anfangswerte setzen
    path=paramsREM.startingWeights;

    if isempty(path)
        Ass_Str = zeros(1, paramsREM.noWeights);
    else % TODO: im Moment fuer jede Vp die gleichen Startgewichte
        Ass_Str=textread(path, '%s', 'bufsize', 100000095, 'delimiter','\n', 'whitespace',' ');
        Ass_Str=str2num(char(Ass_Str(size(Ass_Str,1),:)));
    end

    weights(1,:)=Ass_Str;

    for j = 1:noPat
        inp  = rpatternVp(j,:);

        if paramsREM.rule == 1 %marburger way

            for k = 1:noT
                TestAktivierung(j,k,s) = Ass_Str * paramsREM.testpattern(k, 1:noInp)';
            end
            Output(j,s)   = Ass_Str*inp';
            Fehler=USVP(j)-Output(j,s);
            beta = paramsGen.beta(paramsGen.CS(j,s));

            Ass_Str = Ass_Str + (beta* (Fehler) * (inp > 0));


        elseif paramsREM.rule == 2 %southamptoner way

            for k = 1:noT
                TestAktivierung(j,k,s) = sum(Ass_Str(paramsREM.testpattern(k, 1:noInp)>0));
            end

            Output(j,s)   = sum(Ass_Str(inp>0)); %#ok<*AGROW>
            Fehler=USVP(j)-Output(j,s);
            beta = paramsGen.beta(paramsGen.CS(j,s));

            Ass_Str = Ass_Str + inp.*(beta* (Fehler) * (inp > 0));

        end


        weights(j+1,:)=Ass_Str;

    end
   
    meanWeights=meanWeights+weights;
    OutputVp=Output(:,s)';
    amatrix(:,:,s)=sorttrials(OutputVp, paramsGen.CS(:,s), length(paramsGen.Stimulusname));
end
 amatrix(:,:,s+1)=mean(amatrix,3);

TestAktivierung(:,:,s+1)=mean(TestAktivierung,3);
meanWeights= meanWeights/paramsGen.AnzVp;

%% ploting
disp('ploting ...')

%create string with training stimulus
trainingstring=[];
for ii=1:length(paramsGen.block)
    trainingstring = [trainingstring, strjoin(paramsGen.Stimulusname(paramsGen.block{ii})',  ','),'|'];
end
% create string with parameters
rulestring={'Schultheis et al. 2008'; 'Glautier 2007'};
if length(paramsGen.beta)<=7
titlestring ={['learning rule: ', rulestring{paramsREM.rule},', r = ' , num2str(paramsREM.r),';'];...
    ['\alpha=', mat2str(paramsREM.alpha'),'; \beta=', mat2str(paramsGen.beta),'; phases: ', trainingstring]};
else
    titlestring ={['learning rule: ', rulestring{paramsREM.rule},', r = ' , num2str(paramsREM.r),';'];...
    ['\alpha=', mat2str(paramsREM.alpha'),'; \beta=xxx ; phases: ', trainingstring]};
end

if paramsREM.weightPlot
    plotWeigthCourse(meanWeights(:,:),'associative weights of elements: REM, ', titlestring);
end

if paramsREM.activationPlots
    plotOutputActivation(TestAktivierung(:,:,s+1),'activation by teststimuli: REM ', titlestring,paramsGen.Stimulusname );
%       plotOutputActivation(amatrix(:,:,s+1),'activation by presented stimulus: REm', titlestring, paramsGen.Stimulusname);
end

%% saving

speicherpath=paramsREM.resultsPath;

if ~isempty(speicherpath{1});
    dlmwrite ([speicherpath{1}, '\',speicherpath{2}, '_testactivation.txt'], TestAktivierung(:,:,s+1), '\t') ;
    dlmwrite ([speicherpath{1}, '\',speicherpath{2},'_outputactivation.txt'], amatrix(:,:,s+1), '\t')
    dlmwrite ([speicherpath{1}, '\',speicherpath{2}, '_weights.txt'],meanWeights(:,:),'\t') ;
end

