function paramsRW=trainRW(paramsGen, paramsRW)
%simulation, ploting and saving of the Rescorla Wagner Model and its extensions


% The generela parameters passed by paramsGen are
% Stimulusname: string name of all CS
% Testname:     "
% Kont:         US paired with CS
% KontTest:     "
% beta:         beta of USs
% AnzVp:        number of subjects
% CS:           CS-sequence: noTrial*AnzVp-matrix
% US:           US-sequence: noTrial*AnzVp-matrix
% code:         stimuluscode of all CS: a line defines the occurrence of each single stimulus in a CS (without unique cue or similar)
% testcode:     "
% block:        CS that occur during training , specified by their line number in Stimulusname




% paramsRW:         parameter for Rescorla-Wagner simulation
% UC                compute unique cues
% mUC               use salience equitation of Pearce (1994)
% resultsPath:      directory where resulting matrices are saved
% startingWeights:  txt-file with associative weights whose last line is used as starting
% alpha             vector with alpha parameter
% alphadefault:     default value defined in default.conf
% weightPlot:       checkbox for weightPlot
% activationPlots:  checkbox for activatiaonPlots


% Variables
% code:             stimulus code
% weights:          associative weights of all subjects


% Copyright (C) 2006-2008  Anna Thorwart and Holger Schultheis




% prepare matrices
% all stimuli
if paramsRW.UC == 1 %unique cue hypotheses
    code=expandMatrix_UC(paramsGen.code);
else  % orignal Rescorla-Wagner Model
    code= paramsGen.code;
end

% test stimuli
testcode=code;
noT=size(testcode,1);

% check alpha

NoAlphas_theo=size(code,2); %Anzahl der Alphas, die benï¿½tigt werden
NoAlphas_be=size(paramsRW.alpha,2); %Anzahl der Alphas, die eingegeben sind

if NoAlphas_theo ~=NoAlphas_be
    meldung=['The number of alphas doesn''t fit. There should be ' num2str(NoAlphas_theo) ', but there are ' num2str(NoAlphas_be) '. Do you wish to run the simulation using the default value for all alphas?.'];
    answer= questdlg(meldung, 'Miscount!', 'Yes', 'Cancel', 'Yes');
    if strcmp('Yes', answer);
        paramsRW.alpha=repmat(paramsRW.alphadefault, 1, NoAlphas_theo);
    else
        return
    end
end
alpha = paramsRW.alpha;

%initialize weights matrix
weights=zeros(size(paramsGen.CS,1),size(code,2),paramsGen.AnzVp);


for s=1:paramsGen.AnzVp

    CScodeVp=code(paramsGen.CS(:,s),:);
    USVP=paramsGen.US(:,s);

    % initialize weights

    if isempty(paramsRW.startingWeights) % no starting weights are defined
        Ass_Str =zeros (size(CScodeVp,2),1); % start with zeros
    elseif ~isempty(paramsRW.startingWeights); %TODO; catch missing file
        path=paramsRW.startingWeights;
        startingWeights=dlmread(path); %read in weights matrix
        Ass_Str= startingWeights(end,:)' ;% set last line as starting weights
        weights(1,:,s)=startingWeights(end,:); % write starting weights in weights matrices
    end

    % Festsetzen, ob Unique Cues in den Salienzbruch miteinbezogen werden
    Salienzgewicht=zeros(1,size(CScodeVp,2)); % initialisieren

%     if paramsRW.mUC == 3  % choose if unique cues are included in salience's fraction 
%         Salienzgewicht=ones(1,size(CScodeVp,2)); % alle auf eins setzen
%     elseif paramsRW.mUC ==2
        Salienzgewicht(1:size(paramsGen.code,2))= 1; % nur die Elemente auf eins setzen
%     end

    % training
    Trials=size(CScodeVp,1);
    for j=1:Trials

        for k=1:noT                                                    %Aktivierung bei den verschiednen Testreizen
            TestAktivierung(j,k,s)=testcode(k,:)*Ass_Str   ;            %j trials. k, teststimuli, s subject
        end

        Output(j,s)=CScodeVp(j,:)*Ass_Str;                                    %Aktivierung bei dem gezeigten CS
        Fehler=USVP(j)-Output(j,s);
        beta = paramsGen.beta(paramsGen.CS(j,s));

        if paramsRW.mUC 
            alphagew=alpha/sum(Salienzgewicht.*CScodeVp(j,:).*alpha) ;          %%%%ALPHA
            Ass_Str=Ass_Str+(CScodeVp(j,:)'.*alpha'.*alphagew'*beta*(Fehler));
        else
            Ass_Str=Ass_Str+(CScodeVp(j,:)'.*alpha'*beta*(Fehler));             %Lernen
        end
        weights(j+1,:,s)=Ass_Str';
    end

     amatrix(:,:,s)=sorttrials(Output(:,s)', paramsGen.CS(:,s), length(paramsGen.Stimulusname));

end

%average over subjects
weights(:,:,s+1)= mean(weights,3);
TestAktivierung(:,:,s+1)=mean(TestAktivierung,3);
amatrix(:,:,s+1)=mean(amatrix,3);



% ploting

%create string with training stimulus
trainingstring=[];
for ii=1:length(paramsGen.block)
    trainingstring = [trainingstring, strjoin(paramsGen.Stimulusname(paramsGen.block{ii})', ','),'|'];
end

% create string with parameters
if size(paramsRW.alpha,2)<7 & length(paramsGen.beta) < 7
    titlestring ={['unique cues:' num2str(paramsRW.UC), ',  modified salience:', num2str(paramsRW.mUC),';'];...
        ['\alpha=', mat2str(paramsRW.alpha'),'; \beta=', mat2str(paramsGen.beta), '; phases: ', trainingstring]};
elseif size(paramsRW.alpha,2) >7 & length(paramsGen.beta) <= 7
    titlestring= {['unique cues:' num2str(paramsRW.UC), ',  modified salience:', num2str(paramsRW.mUC),';'];...
        ['\alpha= xxx; \beta=', mat2str(paramsGen.beta),'; phases: ', trainingstring]};
elseif size(paramsRW.alpha,2) <= 7 & length(paramsGen.beta) > 7
    titlestring= {['unique cues:' num2str(paramsRW.UC), ',  modified salience:', num2str(paramsRW.mUC),';'];...
        ['\alpha=', mat2str(paramsRW.alpha'),' \beta=xxx; phases: ', trainingstring]};
else
    titlestring= {['unique cues:' num2str(paramsRW.UC), ',  modified salience:', num2str(paramsRW.mUC),';'];...
        ['\alpha= xxx; \beta=xxx ; phases: ', trainingstring]};
end

if paramsRW.weightPlot
    plotWeigthCourse(weights(:,:,s+1),'associative weights of elements: RW, ', titlestring);
end
if paramsRW.activationPlots
    plotOutputActivation(TestAktivierung(:,:,s+1),'activation by teststimuli: RW ', titlestring,paramsGen.Stimulusname );
%     plotOutputActivation(amatrix(:,:,s+1),'activation by presented stimulus: RW ', titlestring, paramsGen.Stimulusname);
end


%Speichern der Gewichtsmatrix
speicherpath=paramsRW.resultsPath;
if ~isempty(speicherpath{1});
    dlmwrite ([speicherpath{1}, '\',speicherpath{2}, '_testactivation.txt'], TestAktivierung(:,:,s+1), '\t') ;
    dlmwrite ([speicherpath{1}, '\',speicherpath{2}, '_outputactivation.txt'], amatrix(:,:,s+1), '\t')
    dlmwrite ([speicherpath{1}, '\', speicherpath{2}, '_weights.txt'],weights(:,:, s+1),'\t') ;
end




%%% SUBFUNCTIONS

function cs_code = expandMatrix_UC(cs)
%function that expands the CSmatrix computing all combinations of elements
%output is matrix with unique cue code for stimuli
idx = 1:size(cs,2); %number of elements

for i=1:size(cs,1); %number of trials / CS types
    cs_matrix =    cs(i,:); % CS_matrix with elemental units
    for j=2:size(cs,2) % combination of elements
        comb = nchoosek (idx, j);
        dummy=1;
        for k=1:size (comb,2);
            dummy = (dummy .* cs_matrix (comb(:,k)));
        end
        cs_matrix = [cs_matrix dummy];
    end
    cs_code(i,:)=cs_matrix;
end
